
package MT::Plugin::GoogleAnalyticsWidget;

use strict;
use warnings;

use base qw( MT::Plugin );
use MT;

use Data::Dumper;

use MT::Util qw( start_end_day epoch2ts ts2epoch format_ts );
use XML::Simple;

use lib 'lib';
use Analytics;

use vars qw( $VERSION $plugin );
$VERSION = '0.2.41';
$plugin = MT::Plugin::GoogleAnalyticsWidget->new ({
    name        => 'GoogleAnalyticsWidget',
    description => '<__trans phrase="Creates a Google Analytics widget in the blog dashboard. This is modified version by Shinichi Nozawa(http://melonsode.fem.jp/fs/). This widget can be worked in MT4.1.  And functions are extended significantly.">',
    version     => $VERSION,

    author_name => 'Shinichi Nozawa at GENOVa(originaly, Apperceptive, LLC)',
    author_link => 'http://www.genova.co.jp/',

    #blog_config_template    => 'blog_config.tmpl',
	system_config_template    => 'blog_config.tmpl',
    settings    => MT::PluginSettings->new ([
        [ 'analytics_username', { Default => undef, Scope => 'system' } ],
        [ 'analytics_password', { Default => undef, Scope => 'system' } ],
        [ 'analytics_profile_id', { Default => undef, Scope => 'system' } ],
        [ 'analytics_period', { Default => 30, Scope => 'system' } ],
		[ 'analytics_param_updated', {Default => 1, Scope => 'system' } ],
    ]),

	l10n_class => 'GoogleAnalyticsWidget::L10N',

});
MT->add_plugin ($plugin);

sub instance { $plugin }

sub init_registry {
	my $component = shift;
	my $reg ={
		'applications' => {
			'cms' => {
				'blog_stats_tabs' => {
					visit => {
						label => 'Google',
						template => 'blog_stats_visit.tmpl',
						stats => \&init_cms,
					},
				},
			},
		},
		'callbacks' => {
			'MT::App::CMS::template_source.blog_stats' => \&init_cms,
		},
	};
	$component->registry($reg);
}

sub init_cms {
    my $app = shift;
    
    {
        local $SIG{__WARN__} = sub {};
        my $orig_method = \&MT::CMS::Dashboard::generate_dashboard_stats;
        *MT::CMS::Dashboard::generate_dashboard_stats = sub {
            if (!$orig_method->(@_)) {
                return;
            }
            generate_analytics_stats (@_);
        }
    }
}

sub generate_analytics_stats {
    my $app = shift;
    my ($param) = @_;

	$param->{gaw_tmpl_folder} = $app->base."/mt/plugins/GoogleAnalyticsWidget/mt-static/";

	#necessary to work without update
	my @graphs = ( 'VisitorsSparkline' ,'VisitsSparkline' ,'PageviewsSparkline' ,'AvgPageviewsSparkline');
	$param->{google_analytics_graphs} = \@graphs;

	if($app->blog){
	#when system menu , nothing is showed
		delete $param->{analytics_visits_stat_url};
		return 1;
	}else{
		#get static url
		my $stat_url = $param->{stat_url}->{comment};
		$stat_url =~ s{comment\.xml$}{};
		$param->{analytics_visits_stat_url} = $stat_url;
	}
	
	if(!$plugin->get_config_value ('analytics_username', 'system')){
	#if id is not set, only panel is appeard
		$param->{gaw_active} = "unset";
		return 1;
	}

	#make Agent and login 
	my $a = Analytics->new;
	my $login = $a->login (
		$plugin->get_config_value ('analytics_username', 'system'), 
		$plugin->get_config_value ('analytics_password', 'system')
	);
	#return 1;
	if(!$login){
		$param->{gaw_active} = "loginFailed";
		return 1;
	}else{
		$param->{gaw_active} = "active";
	}

	#check necessity of update
	my $xml_ref = &getXMLRef($a,  'VisitorsOverviewReport', $app);
	$param->{gaw_profilename} = $xml_ref->{Report}->{Title}->{ProfileName};
	my $update = &needUpdate($app, $param, 'VisitorsOverviewReport', $graphs[0]);
	if(!$update){
		$param->{gaw_update} = " are up to date";
		#return 1;
	}else{
		$param->{gaw_update} = " have been updated";
	}

	#make a ranking part
	my @words = ('ContentByTitle', 'Keywords', 'ReferringSources');
	my $days = $plugin->get_config_value ('analytics_period', 'system');
	if ($days > 120){
		$days = 120;
		#$plugin->set_config_value('analytics_period', 'blog:' . $app->blog->id);
	}
	my $period = 60 * 60 * 24*$days;#60 * 60 * 24 * 12;
	$param->{gaw_period} = $days;#$plugin->get_config_value('analytics_period', 'blog:' . $app->blog->id);
	$param->{gaw_profile_id} = $plugin->get_config_value ('analytics_profile_id', 'system');
	foreach (@words){
		my $var_name = "gaw_".$_;
		my $file_name = $_."Report";
		$param->{$var_name} = &generate_words_variants($app, $param, $file_name, $a, $period);
	}

	#make a graph part
	if(!$update){
		return 1;
	}

	foreach (@graphs) {
		&generate_sparkline_xml($app, $param,  $_, $xml_ref);
	}

    1;
}

sub generate_words_variants {
    my $app = shift;
    my $param = shift;
	my $fn = shift;#filename
	my $a = shift;#UserAgent
	my $period = shift;#60 * 60 * 24 * 120;
	my $now = time;
	my @words;

	#get xml at google
	my $today = start_end_day ( epoch2ts ($app->blog, $now ) );
	my $thirty_days_ago = start_end_day ( epoch2ts ($app->blog, $now - $period) );
	my $xml_ref = $a->getReport (
		$plugin->get_config_value ('analytics_profile_id', 'system'), 
		format_ts ('%Y%m%d', $thirty_days_ago, $app->blog), 
		format_ts ('%Y%m%d', $today, $app->blog), 
		$fn
	);

	#set the data in MT
	my $Report = $xml_ref->{Report}->{Table}->{Row};
	my $all_page_views = $xml_ref->{Report}->{Table}->{Column}[0]->{Total};
	$all_page_views =~ s/,//g;

	if(ref($Report) ne "ARRAY"){
		return 1;
	}

	foreach (@$Report){
		my $rate = $_->{Cell}[0]->{Content}->{Value};
		$rate =~ s/,//g;
		$rate = int( ($rate/$all_page_views)*100 + 0.5 );
		push @words, {
			'title' => $_->{Key},
			'PageView' => $_->{Cell}[0]->{Content}->{Value},
			'rate' => $rate,
		};
	}

	return \@words;

}

sub getXMLRef{
	my $a = shift;
	my $fn = shift;
	my $app = shift;
	
	my $now = time;
	my $today = start_end_day ( epoch2ts ($app->blog, $now ) );
	my $thirty_days_ago = start_end_day ( epoch2ts ($app->blog, $now - (60 * 60 * 24 * 120)) );
	
	my $xml_ref = $a->getReport (
		$plugin->get_config_value ('analytics_profile_id', 'system'), 
		format_ts ('%Y%m%d', $thirty_days_ago, $app->blog), 
		format_ts ('%Y%m%d', $today, $app->blog), 
		$fn
	);
	
	return $xml_ref;
}


sub generate_sparkline_xml {
    my $app = shift;
    my $param = shift;
	my $phrase = shift;
	my $xml_ref = shift;
	
	my $analytics_visits_path = File::Spec->catfile ($param->{support_path}, $phrase.'.xml');

	my $now = time;
	my $today = start_end_day ( epoch2ts ($app->blog, $now ) );
	my $thirty_days_ago = start_end_day ( epoch2ts ($app->blog, $now - (60 * 60 * 24 * 120)) );
	
	#make formed statistic data
	my @data = @{$xml_ref->{Report}->{Sparkline}->{$phrase}->{PrimaryValue}};
	my $this_day = $thirty_days_ago;
	my %counts = ();
	foreach my $count (@data) {
		$counts{substr ($this_day, 0, 8) . 'T00:00:00'} = $count;
		$this_day = epoch2ts ( $app->blog, ts2epoch ($app->blog, $this_day) + (60 * 60 * 24));
	}
        
	#put %count to make the local xml file
	unless ($app->create_dashboard_stats_file ($analytics_visits_path, \%counts)) {
		delete $param->{analytics_visits_stat_url};
	}

}


sub needUpdate {
    my $app = shift;
    my $param = shift;
	my $fn = shift;
	my $phrase = shift;
	my $cache_time = 60 * 60 * 24; # cache for one day

	#when plugin settings are changed
	if( $plugin->get_config_value ('analytics_param_updated', 'system') ){
		$plugin->set_config_value ('analytics_param_updated', 0, 'system');
		return 1;
	}
	
	my $analytics_visits_path = File::Spec->catfile ($param->{support_path}, $phrase.'.xml');

    my $analytics_visits_time = ( stat($analytics_visits_path) )[9] if -f $analytics_visits_path;

	#check the cache
    if ( (!$analytics_visits_time ) || ( time - $analytics_visits_time > $cache_time )) {
		return 1;
    }else{
		return 0;
	}

}



1;
